/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.bck2brwsr.demo.serverside;

import net.java.html.json.ComputedProperty;
import net.java.html.json.Context;
import net.java.html.json.Function;
import net.java.html.json.Model;
import net.java.html.json.OnReceive;
import net.java.html.json.Property;

/**
 *
 * @author Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 */
@Model(className = "ChatModel", properties = {
    @Property(name = "user", type = String.class),
    @Property(name = "comment", type = String.class),
    @Property(name = "msgs", type = Message.class, array = true)
})
class ChatClient {
    @ComputedProperty
    static boolean sendEnabled(String user, String comment) {
        boolean res = user != null && comment != null && !user.isEmpty() && !comment.isEmpty();
        return res;
    }
    
    @Function
    static void submit(ChatModel m) {
        if (!sendEnabled(m.getUser(), m.getComment())) {
            return;
        }
        m.postComment(m.getUser(), m.getComment());
    }
    
    @OnReceive(url = "/chat/addComment?user={user}&comment={comment}")
    static void postComment(ChatModel m, Message addedMessage) {
        if (addedMessage.getComment().equals(m.getComment())) {
            m.setComment("");
        }
    }
    
    @OnReceive(url = "/chat?since=0")
    static void initialRead(ChatModel m, Query q) {
        m.getMsgs().clear();
        m.getMsgs().addAll(q.getMessages());
        moreMessages(m);
    }
    
    @OnReceive(url = "/chat?since={since}")
    static void updateMsgs(ChatModel m, Query q) {
        m.getMsgs().addAll(q.getMessages());
        moreMessages(m);
    }
    
    private static void moreMessages(ChatModel m) {
        long now = System.currentTimeMillis();
        m.updateMsgs("" + now);
    }
    
    static final Context CNTX = Context.findDefault(ChatClient.class);
    static {
        ChatModel chm = new ChatModel(CNTX);
        Message m = new Message(CNTX);
        m.setComment("Waiting for messages from the server...");
        m.setUser("system");
        chm.getMsgs().add(m);
        chm.applyBindings();
        try {
            // XXX: this should not be in static initializer -
            // XXX: prevents unit testing!
            chm.initialRead();
        } catch (Throwable ex) {
        }
    }
}
